% Attitude and Position of Aircraft
p_aircraft_n = [100 0 -100].';
Theta_nb     = [0 0    0].';

% Attitude and position of Gimbal
p_gimbal_b  = [0 0    0].';
Theta_mount = [0 0    0].';
Theta_cam   = [0 0    0].';

R_NED_BODY = @(a) Rzyx(a(1), a(2), a(3));
R_BODY_CAM = @(c) Rzyx(0, c(2), c(3)) * Rzyx(Theta_mount(1), Theta_mount(2), Theta_mount(3));
R_NED_CAM  = @(a, c) R_BODY_CAM(c) * R_NED_BODY(a).';

PLOT2      = @(v,args) plot3(v(1), v(2), 0, args);
PLOTLINE   = @(v1,v2)  plot3([v1(1), v2(1)], [v1(2) v2(2)], [-v1(3) -v2(3)]);
PLOT3      = @(v,args) plot3(v(1), v(2), -v(3), args);

f = 9*10^-3 / (17*10^-6);
k_u = 704;
k_v = 480;
u_0 = k_u/2;
v_0 = k_v/2;
A = [f 0 u_0; 0 f v_0; 0 0 1];
%A = [4.73*10^3 0 3.69*10^2;
%     0 4.73*10^3 2.61*10^2;
%     0 0 1];

k_1 = 0.4;
k_2 = 4;
k_3 = 0;%-1.43*10^-1;
k_4 = 0;%-1.7*10^-2;
k_5 = 0;% -3.1*10^3;

% Create some ships
ships = zeros(2,200);
for i=1:14
    for j=1:14
        ships(:,(i*20)+j) = [j*5 - 37.5, i*5 - 37.5];
    end
end

f     = @(c, d) [c(6)*cos(c(4));
                 c(6)*sin(c(4));
                              0;
          0;%(9.81/c(6))*sin(c(5));
                              0;
                              0];
x = [0 0 -100 0 0 0].';

h = 1;
for i=1:1
    x_dot = f(x,[0 0]);
    x = x + h*x_dot
    
    p_aircraft_n = [x(2) x(1) x(3)].';
    Theta_nb = [x(5) 0 x(4)].';
    R = R_NED_CAM(Theta_nb, Theta_cam);
    c1 = R(:,1);
    c2 = R(:,2);

    %F = [1 0 0; 0 -1 0; 0 0 1];
    X = [c1 c2 R*p_aircraft_n];
    D = A * X;
    
    invD = inv(D);
    % Plot the aircraft
    figure(1);
    PLOT3(p_aircraft_n,'o');
    clf
    hold on
    grid on

    % Plot the four corners of the image
    tl = invD*[0     0 1].';
    tl = tl / tl(3)
    tr = invD*[k_u   0 1].';
    tr = tr / tr(3)
    bl = invD*[0   k_v 1].';
    bl = bl / bl(3);
    br = invD*[k_u k_v 1].';
    br = br / br(3);

    PLOT2(tl,'ro');
    PLOTLINE(tl,tr);
    PLOTLINE(tl,p_aircraft_n);
    PLOT2(tr,'go');
    PLOTLINE(tr,br);
    PLOTLINE(tr,p_aircraft_n);
    PLOT2(bl,'o');
    PLOTLINE(br,bl);
    PLOTLINE(br,p_aircraft_n);
    PLOT2(br,'co');
    PLOTLINE(bl,tl);
    PLOTLINE(bl,p_aircraft_n);

    % Draw the ships in the 3D plot
    for n=1:length(ships)
        ship = ships(:,n);
        PLOT2(ship(1:2), 'xk');
    end

    %set(gca, 'Ydir', 'Reverse');

    axis([-100 100 -100 100 0 100]);
    xlabel('East [m]');
    ylabel('North [m]');
    zlabel('Altitude [m]');
 
    hold off
    
    figure(2);
    clf
    
    hold on
    for n=1:length(ships)
        ship = ships(:,n);
        shipPos = X*[ship; 1];
        shipPos = shipPos / shipPos(3);
        r = sqrt(shipPos(1)^2 + shipPos(2)^2);
        e = 1 + k_1*r^2 + k_2*r^4 + k_3*r^6;
        p = A*[shipPos(1)*(e); shipPos(2)*(e); 1];
        if p(1) >= 0 && p(1) <= k_u && p(2) >= 0 && p(2) <= k_v
            plot(p(1), p(2),'xk', 'MarkerSize', 12);
        end
    end
    hold off
    set(gca, 'Ydir', 'Reverse');
    xlabel('x axis [pixels]');
    ylabel('y axis [pixels]');
    axis([0 k_u 0 k_v]);
    %drawnow
    %pause(0.001);
    M(i) = getframe(gcf);
end






