%%%%% Ship Model %%%%%
x     = zeros(6,1);
x_dot = zeros(6,1);

m     = 2000; % kg
Dv    = 1000;  % No idea
Dp    = 1000;  % No idea
D     = Dp + Dv;
C     = 10;

f     = @(c, d) [c(4)*cos(c(3)) - c(5)*sin(c(3));
                 c(4)*sin(c(3)) + c(5)*cos(c(3));
                                            c(6);
                               (d(1) - D*c(4))/m;
                                     (-D*c(5))/m;
                         c(4)*sin(d(2)) - C*c(6);];
                              
s = [4,0].';

%%%%% Simulation parameters %%%%%
h = 0.01; % Step size

%%%%% Extended Kalman filter %%%%%
Q     = 10*diag([0.01 0.01 0.01 0.01 0.01 0.01]);
%Q     = diag([1 1 1 1 1 1]);
x_bar = [0 0 0 0 0 0].';
P_bar = diag([0 0 0 0 0 0]);

%%%%% Yaw feedback %%%%%
%R = diag([2.07 2.07 0.0033 10^15 10^15 10^15]);
R = diag([2.07 2.07 8.4*10^-4]);
H = [1 0 0 0 0 0;
     0 1 0 0 0 0;
     0 0 1 0 0 0];

%%%%% No yaw feedback %%%%%
%R = diag([2.07 2.07 10^10 10^10 10^10 10^10]);
%H = diag([1 1 0 0 0 0]);

%%%%% No noise %%%%%
%R = diag([0.1 0.1 0.1]);
%H = [1 0 0 0 0 0;
%     0 1 0 0 0 0;
%     0 0 1 0 0 0];

E = diag([1 1 1 1 1 1]);

Gamma = h*E;
K     = zeros(6,3);

% Measurement
y = zeros(3,1);

%%%%% Kalman Model %%%%
f_k   = @(c) [c(4)*cos(c(3)) - c(5)*sin(c(3));
              c(4)*sin(c(3)) + c(5)*cos(c(3));
                                         c(6);
                                            0;
                                            0;
                                            0;];
                     
df_k = @(c) [0 0 (-c(4)*sin(c(3)) + c(5)*cos(c(3))) cos(c(3)) -sin(c(3)) 0;
             0 0 ( c(4)*cos(c(3)) - c(5)*sin(c(3))) sin(c(3))  cos(c(3)) 0;
             0 0                                  0         0          0 1;
             0 0                                  0         0          0 0;
             0 0                                  0         0          0 0;
             0 0                                  0         0          0 0;];

%%%%% Model simulation %%%%%
x_out_m = zeros(7,6000);
x_out_k = zeros(7,6000);
figure(1);
clf;
hold on;
u_prev = [0 0].';
u = [0 0].';

for t=1:6000
    %%%%% Model %%%%%
    if t==1000
        s = [4;pi/2];
    end
    if t==2500
        s = [5;-pi/2];
    end
    
    x_dot = f(x,u);
    
    x = x + h*x_dot;
    
    u1 = u_prev(1) + 10*(s(1)-x(4));
    u2 = 1*(s(2)-x(3));
    u = [u1, u2].';
    u_prev = u;

    x_out_m(1:6,t) = x;
 
    %%%%% Kalman %%%%%
    if mod(t,13) == 0 || t == 0
        if t>1000 && t < 1200
            y = zeros(3,1);
            K = zeros(6,3);
        elseif t>1000 && t < 1200
            y = zeros(3,1);
            K = zeros(6,3);
        elseif t>1800 && t < 2500
            y = zeros(3,1);
            K = zeros(6,3);  
        elseif t>3800 && t < 4100
            y = zeros(3,1);
            K = zeros(6,3);  
        else
            y = [x(1)+(5*rand()-2.5) x(2)+(5*rand()-2.5) x(3)+(0.05*rand()-0.025)].';
            %y = [x(1) x(2) x(3)].';
            K = P_bar*H.' * inv(H*P_bar*H.' + R);
        end
    end
    
    x_hat = x_bar + K * (y - H*x_bar);
    
    P_hat = (eye(6) - K*H)*P_bar*(eye(6)-K*H).' + K*R*K.';
    
    x_bar = x_hat + h*(f_k(x_hat));
    Phi = eye(6) + h*df_k(x_bar);
    
    P_bar = Phi*P_hat*Phi.' + Gamma*Q*Gamma.';
    
    x_out_k(1:6,t) = x_hat;
    x_out_k(7,t) = sqrt(x_hat(4)^2 + x_hat(5)^2);
    x_out_m(7,t) = sqrt(x(4)^2 + x(5)^2);
    
    %%%%% Plotting %%%%%
    hold on
    
    plot(x(2), x(1), 'k', 'MarkerSize', 4);
    plot(y(2), y(1), 'ob', 'MarkerSize', 3);
    plot(x_hat(2), x_hat(1), 'r', 'LineWidth', 2);
end

% Set axes of left display window
axis([-10 120 0 85]);
xlabel('East [m]');
ylabel('North [m]');
legend('Actual ship position', 'Measurements', 'Estimated position');

%%%%% Plot after simulation %%%%%
hold off
figure(2);
clf
hold on
plot(abs((x_out_k(3,:) - x_out_m(3,:))*(180/pi)),'k');
xlabel('Time [ms]');
ylabel('Degrees');
legend('Yaw error');
hold off
figure(3);
clf
hold on
plot(abs(x_out_k(4,:) - x_out_m(4,:)), 'k');
plot(x_out_m(4,:), 'r');
plot(x_out_k(4,:), 'g');
xlabel('Time [ms]');
ylabel('Speed [m/s]');
legend('Forward speed (u) error');
hold off
figure(4);
clf
hold on
plot(abs(x_out_k(5,:) - x_out_m(5,:)), 'k');
xlabel('Time [ms]');
ylabel('Speed [m/s]');
legend('Lateral speed (v) error');
hold off
figure(5);
clf
hold on
plot(abs(x_out_k(6,:) - x_out_m(6,:)), 'k');
xlabel('Time [ms]');
ylabel('rad/s');
legend('Yaw rate error');
hold off
figure(6);
clf
hold on
plot(abs(x_out_k(7,:)-x_out_m(7,:)),'k');
xlabel('Time [ms]');
ylabel('Speed [m/s]');
legend('Speed error (Total speed)');
hold off
figure(7);
clf
hold on
plot(abs(x_out_k(1,:)-x_out_m(1,:)),'k');
hold off
xlabel('Time [ms]');
ylabel('Distance [m]');
legend('Position error (N)');
figure(8);
clf
hold on
plot(abs(x_out_k(2,:)-x_out_m(2,:)),'k');
hold off
xlabel('Time [ms]');
ylabel('Distance [m]');
legend('Position error (E)');