%%%%% Ship Model %%%%%
x     = zeros(6,1);
x_dot = zeros(6,1);

m     = 2000; % kg
Dv    = 1000;  % No idea
Dp    = 1000;  % No idea
D     = Dp + Dv;
C     = 10;

f     = @(c, d) [c(4)*sin(c(3)) - c(5)*cos(c(3));
                 c(4)*cos(c(3)) + c(5)*sin(c(3));
                                            c(6);
                               (d(1) - D*c(4))/m;
                                     (-D*c(5))/m;
                         c(4)*sin(d(2)) - C*c(6);];
                              
u = [10000; 0];

%%%%% Simulation parameters %%%%%
h = 0.01; % Step size

%%%%% Linear Kalman filter %%%%%
A = [0 0 1 0;
     0 0 0 1;
     0 0 0 0;
     0 0 0 0];

B = [0 0 0 0].';

[Phi,Delta] = c2d(A,B,h);

H = [1 0 0 0;
     0 1 0 0;
     0 0 0 0;
     0 0 0 0];

P_bar = [0 0 0 0;
         0 0 0 0;
         0 0 0 0;
         0 0 0 0];
     
R = diag([2.07 2.07 10^15 10^15]); 
Q = diag([0.5, 0.5, 0.005, 0.005]);

K = zeros(4,4);

x_bar = [0 0 0 0].';
x_hat = x_bar;
y     = x_hat;

E = diag([1 1 1 1]);

Gamma = (eye(4)*h + 0.5*A*h^2)*E;

%%%%% Model simulation %%%%%
x_out_m = zeros(6,4000);
x_out_k = zeros(4,4000);
figure(1);
clf;
hold on;

for t=1:4000
    %%%%% Model %%%%%
    if t==1400
        u = [10000;-1];
    end
    if t==2800
        u = [10000;0];
    end
    
    x_dot = f(x,u);
    
    x = x + h*x_dot;
    
    x_out_m(1:6,t) = x;
    
    %%%%% Kalman %%%%%
    if mod(t,13) == 0
        y = [x(1)+(5*rand() - 2.5) x(2)+(5*rand() - 2.5) 0 0].';
        K = P_bar*H.' * inv(H*P_bar*H.' + R);
    end
    
    x_hat = x_bar + K * (y - H*x_bar);
    P_hat = (eye(4) - K*H)*P_bar*(eye(4)-K*H).' + K*R*K.';
    
    x_bar = Phi*x_hat;
    P_bar = Phi*P_hat*Phi.' + Gamma*Q*Gamma.';
    
    %%%%% Plotting %%%%%
    hold on
    
    plot(x(1), x(2));
    plot(y(1),y(2),'og');
    plot(x_hat(1), x_hat(2), 'or');
    
    x_out_k(1:4,t) = x_hat;
end

%%%%% Plot after simulation %%%%%
hold off
figure(2);
clf
subplot(2,2,1);

hold on
plot(x_out_k(1,:)-x_out_m(1,:),'r');
hold off
subplot(2,2,2);
hold on
plot(x_out_k(2,:)-x_out_m(2,:),'r');
hold off
subplot(2,2,3);
hold on
plot(x_out_k(3,:),'g');
hold off
subplot(2,2,4);
hold on
plot(x_out_k(4,:),'g');
hold off
