%%%%% Ship Model %%%%%
x     = zeros(6,1);
x_dot = zeros(6,1);

m     = 2000; % kg
Dv    = 1000;  % No idea
Dp    = 1000;  % No idea
D     = Dp + Dv;
C     = 10;

f     = @(c, d) [c(4)*cos(c(3)) - c(5)*sin(c(3));
                 c(4)*sin(c(3)) + c(5)*cos(c(3));
                                            c(6);
                               (d(1) - D*c(4))/m;
                                     (-D*c(5))/m;
                         c(4)*sin(d(2)) - C*c(6);];
                              
s = [4,0].';

%%%%% Simulation parameters %%%%%
h = 0.01; % Step size

%%%%% Linear Kalman filter %%%%%
A = [0 0 1 0;
     0 0 0 1;
     0 0 0 0;
     0 0 0 0];

B = [0 0 0 0].';

[Phi,Delta] = c2d(A,B,h);

H = [1 0 0 0;
     0 1 0 0];

P_bar = [0 0 0 0;
         0 0 0 0;
         0 0 0 0;
         0 0 0 0];

%R = diag([2.07 2.07]); 
%Q = 0.5*eye(4);
R = diag([0.1 0.1]);
Q = diag([0.1 0.1 0.1 0.1]);
K = zeros(4,2);

x_bar = [0 0 0 0].';
x_hat = x_bar;
y     = [0 0].';

E = diag([1 1 1 1]);

Gamma = (eye(4)*h + 0.5*A*h^2)*E;

%%%%% Model simulation %%%%%
x_out_m = zeros(7,4000);
x_out_k = zeros(5,4000);
figure(1);
clf;
hold on;

u_prev = [0 0].';
u      = [0 0].';

l = 0;

for t=1:6000
    %%%%% Model %%%%%
    if t==1000
        s = [4;pi/2];
    end
    if t==2500
        s = [5;-pi/2];
    end
    
    x_dot = f(x,u);

    u1 = u_prev(1) + 10*(s(1)-x(4));
    u2 = 1*(s(2)-x(3));
    
    if u1>10000
        u1 = 10000;
    end
    if u1<0
        u1 = 0;
    end
    
    u = [u1, u2].';
    
    x = x + h*x_dot;
    
    x_out_m(1:6,t) = x;
    
    u_prev = u;
    %%%%% Kalman %%%%%
    if mod(t,13) == 0
        %y = [x(1)+(5*rand() - 2.5) x(2)+(5*rand() - 2.5)].';
        y = [x(1) x(2)].';
        S = H*P_bar*H.' + R;
        S = 0.5*S + 0.5*S.';
        l = [l; det(S)];
        
        K = P_bar*H.' * pinv(S);
    end
    
    x_hat = x_bar + K * (y - H*x_bar);
    P_hat = (eye(4) - K*H)*P_bar*(eye(4)-K*H).' + K*R*K.';
    %P_hat = P_bar - P_bar*H.'*inv(H*P_bar*H.'+R)*H*P_bar;
    %P_hat = P_bar - K*(H*P_bar*H.'+R)*K.';
    %P_hat = (eye(4) - K*H)*P_bar;
    
    x_bar = Phi*x_hat;
    P_bar = Phi*P_hat*Phi.' + Gamma*Q*Gamma.';
    
    %%%%% Plotting %%%%%
    hold on
    
    plot(x(2), x(1), 'k');
    plot(y(2),y(1), 'ob', 'MarkerSize', 2);
    plot(x_hat(2), x_hat(1), 'r', 'LineWidth', 2);
    
    x_out_k(1:4,t) = x_hat;
    x_out_k(5,t) = sqrt(x_hat(3)^2 + x_hat(4)^2);
    x_out_m(7,t) = sqrt(x(4)^2 + x(5)^2);
end

axis([-10 120 0 85]);
xlabel('East [m]');
ylabel('North [m]');
legend('Actual ship position', 'Measurements', 'Estimated position');

%%%%% Plot after simulation %%%%%
hold off
figure(2);
clf
hold on
plot(abs(x_out_k(1,:)-x_out_m(1,:)),'k');
hold off
xlabel('Time [ms]');
ylabel('Distance [m]');
legend('Position error (N)');
figure(3);
clf
hold on
plot(abs(x_out_k(2,:)-x_out_m(2,:)),'k');
hold off
xlabel('Time [ms]');
ylabel('Distance [m]');
legend('Position error (E)');
figure(4);
clf
hold on
plot(abs(x_out_k(5,:)-x_out_m(7,:)),'k');
xlabel('Time [ms]');
ylabel('Speed [m/s]');
legend('Speed error (Total speed)');
hold off
